/**
* Filename: IncomingMessageQueue.java (in org.openbandy.net.client)
* This file is part of the BandyBasics project.
*
* www.openbandy.org
*/

package org.openbandy.net.client;

import java.util.Vector;

import org.openbandy.io.Message;
import org.openbandy.service.LogService;

/**
 * Local Queue for Messages. Every Message will be assigned a unique integer
 * which allows to dequeue Messages identified by this integer.
 * If MESSAGE_MAX is reached, the 5 oldest Message will be deleted.
 * 
 * <br><br>
 * (c) Copyright Philipp Bolliger 2007, ALL RIGHTS RESERVED.
 *
 * @author Philipp Bolliger (philipp@bolliger.name)
 * @version 0.1
 */

public class IncomingMessageQueue {
    
    /* Constant :  maximum of messages allowed */
    private final static int MESSAGE_MAX = 20;
    
    /* Queue of Messages that need to be delivered */
    private Vector messageQueue = new Vector();
    
    /* List of free identifiers */
    private Vector freeIdentifiers = new Vector();
    
    /* List of identifiers in the order of insertion */
    private Vector insertionOdrder = new Vector();
    
    /* index of last element inserted */
    private int numberOfMessageInQueue = 0;
    
    public int enqueue(Message message){
        int insertedAt = 0;
        
        /* check size and remove old messages if necessary */
        if(numberOfMessageInQueue == MESSAGE_MAX){
            removeOldestMessages();
        }
        /* insert in queue */
        if(freeIdentifiers.size()>0){
            insertedAt = ((Integer)freeIdentifiers.elementAt(0)).intValue();
            freeIdentifiers.removeElementAt(0);
            messageQueue.setElementAt(message, insertedAt);
        }else{
            messageQueue.addElement(message);
            insertedAt = messageQueue.indexOf(message);
        }
        
        /* remember insertion order */
        insertionOdrder.addElement(new Integer(insertedAt));
        
        LogService.debug(this, "Enqueued message at position "+insertedAt);
        numberOfMessageInQueue++;
        return insertedAt;
    }
    
    public Message dequeue(int identifier){
        if( (identifier < 0) || (identifier >= messageQueue.size()) ){
            LogService.warn(this, "Invalid identifier");
            return null;
        }else{
            /* dequeue message */
            Message m = (Message)messageQueue.elementAt(identifier);
            messageQueue.setElementAt(null, identifier);
            freeIdentifiers.addElement(new Integer(identifier));
            
            /* maintain insertion order */
            insertionOdrder.removeElement(new Integer(identifier));
            
            LogService.debug(this, "Dequeued message at position "+identifier);
            numberOfMessageInQueue--;
            return m;
        }
    }
    
    /* ************* Helper Methods ************** */
    private void removeOldestMessages(){
    	LogService.info(this, "Removing 5 oldest Messages");
        System.out.println("REMOVING 5 OLDEST");
        for(int i=0;i<5;i++){
            Integer removeIdentifier = (Integer)insertionOdrder.firstElement();
            insertionOdrder.removeElementAt(0);
            dequeue(removeIdentifier.intValue());
        }
    }
    
    //DEV PURPOSE
//    protected void print() {
//        System.out.println("--size : "+messageQueue.size()+" numberOfM : "+numberOfMessageInQueue);
//        System.out.println("QUEUE ----------");
//        for(int i=0; i<messageQueue.size(); i++){
//            Message m = (Message)messageQueue.elementAt(i);
//            if(m == null){
//                System.out.println(i+" is null");
//            }else{
//                System.out.println(i+" "+m.getLegibleString());
//            }
//        }
//        System.out.println("ORDER ----------");
//        for(int i=0; i<insertionOdrder.size(); i++){
//            Integer in = (Integer)insertionOdrder.elementAt(i);
//            System.out.println(i+" "+in);
//        }
//    }

}
