package org.openbandy.service;

import org.openbandy.log.Log;
import org.openbandy.log.LogConfiguration;
import org.openbandy.log.LogImpl;


/**
 * Helper class to simplify the usage of logging methods
 * as logging functionality con be used statically via
 * the methods provided in this class.
 * 
 * @version 1.5
 * @author Philipp Bolliger (philipp@bolliger.name)
 */
public final class LogService extends Service {

	/* static reference to the log */
	private static Log log;
	
	/* try to statically assign a log of type
	 * org.openbandy.log.LogImpl */
	static{
		String conf = System.getProperty("microedition.configuration");
	    if(conf != null){
	    	String profiles = System.getProperty("microedition.profiles");
	    	String platform = System.getProperty("microedition.platform");
	    	String encoding = System.getProperty("microedition.encoding");
	    	String locale = System.getProperty("microedition.locale");
	    	
	    	log = new LogImpl(LogConfiguration.getLogAsAppScreenConfig(true));
	    	debug("LogService", "Running on "+conf+" "+profiles+" "+platform+" "+encoding+" "+locale);
	    }
	}


	/* ***** Getter and Setter Methods  ***** */

	/**
	 * Returns a reference to the actual log if this is
	 * an instance of type <code>org.openbandy.log.LogImpl</code>.
	 *
	 * @return Reference to the current log if it is a MicroeditionLog, null else
	 */
	public static LogImpl getLog(){
		try {
			LogImpl microeditionLog = (LogImpl)log;
			return microeditionLog;
		} catch (ClassCastException cce) {
			error("LogService", "Log is not a MicroeditionLog", cce);
			return null;
		}
	}

	public static void setLog(Log log){
		LogService.log = log;
	}


	/* ***** Logging Methods ***** */

	/**
	 * Forwards a message with log level LEVEL_ERROR to the log service
	 * 
	 * @param origin The object reporting
	 * @param msg	 The message to log
	 * @param exception The exception that was caused with this error
	 */
	public static void error(Object origin, String msg, Throwable exception) {
		if(log != null){
			log.error(origin, msg, exception);
		}
	}

	/**
	 * Forwards a message with log level LEVEL_ERROR to the log service
	 * 
	 * @param originClassName The reporting object's class name
	 * @param msg	 The message to log
	 * @param exception The exception that was caused with this error
	 */
	public static void error(String originName, String msg, Throwable exception) {
		if(log != null){
			log.error(originName, msg, exception);
		}
	}

	/**
	 * Forwards a message with log level LEVEL_WARNING to the log service
	 * 
	 * @param origin The object reporting
	 * @param msg	 The message to log
	 */
	public static void warn(Object origin, String msg) {
		if(log != null){
			log.warn(origin, msg);
		}
	}
	
	/**
	 * Forwards a message with log level LEVEL_WARNING to the log service
	 * 
	 * @param origin The name of the object reporting
	 * @param msg	 The message to log
	 */
	public static void warn(String originName, String msg) {
		if(log != null){
			log.warn(originName, msg);
		}
	}

	/**
	 * Forwards a message with log level LEVEL_INFO to the log service
	 * 
	 * @param origin The object reporting
	 * @param msg	 The message to log
	 */
	public static void info(Object origin, String msg) {
		if(log != null){
			log.info(origin, msg);
		}
	}
	
	/**
	 * Forwards a message with log level LEVEL_INFO to the log service
	 * 
	 * @param origin The name of the object reporting
	 * @param msg	 The message to log
	 */
	public static void info(String originName, String msg) {
		if(log != null){
			log.info(originName, msg);
		}
	}

	/**
	 * Forwards a message with log level LEVEL_DEBUG to the log service
	 * 
	 * @param origin The object reporting
	 * @param msg	 The message to log
	 */
	public static void debug(Object origin, String msg) {
		if(log != null){
			log.debug(origin, msg);
		}
	}
	
	/**
	 * Forwards a message with log level LEVEL_DEBUG to the log service
	 * 
	 * @param origin The name of the object reporting
	 * @param msg	 The message to log
	 */
	public static void debug(String originName, String msg) {
		if(log != null){
			log.debug(originName, msg);
		}
	}
	
	/**
	 * Set the log level. 
	 *
	 * @param level Log level according to @link org.openbandy.log.LogLevel
	 */
	public static void setLogLevel(int level){
		if(log != null){
			log.setLogLevel(level);
		}
	}
	
}
